# -*- coding: utf-8 -*-
"""
Module for plotting Bunch and Beam objects.

@author: Watanyu Foosang
@Date: 10/04/2020
"""

import numpy as np
import matplotlib.pyplot as plt
import seaborn as sns
import h5py as hp


def plot_bunchdata(filename, bunch_number, dataset, option=None, x_var="time"):
    """
    Plot the evolution of the variables from the Beam object.
    
    Parameters
    ----------
    filename : str
        Name of the HDF5 file that contains the data.
    bunch_number : int
        The bunch number whose data has been saved in the HDF5 file.
        This has to be identical to 'bunch_number' parameter in 'BunchMonitor' object.
    detaset : str {"current","emit","mean","std"}
        HDF5 file's dataset to be plotted.
    option : str, optional
        If dataset is "emit", "mean", or "std", the variable name to be plotted
        needs to be specified :
            for "emit", option = {"x","y","s"}
            for "mean" and "std", option = {"x","xp","y","yp","tau","delta"}    
    x_var : str, optional
        Variable to be plotted on the horizontal axis. The default is "time".

    """
    
    file = hp.File(filename, "r")
    
    group = "BunchData_{0}".format(bunch_number)  # Data group of the HDF5 file
    
    if dataset == "current":
        y_var = file[group][dataset][:]*1e3
        label = "current (mA)"
        
    elif dataset == "emit":
        # Specifying the axis
        # horizontal axis (x)   -> 0 
        # vertical axis (y)     -> 1 
        # longitudinal axis (s) -> 2 
                         
        emit_axis = {"x":0, "y":1, "s":2}
                         
        y_var = file[group][dataset][emit_axis[option]]*1e9
        
        if option == "x": label = "hor. emittance (nm.rad)"
        elif option == "y": label = "ver. emittance (nm.rad)"
        elif option == "s": label = "long. emittance (nm.rad)"
        
        
    elif dataset == "mean" or "std":
        # Specify the variable
        # x     -> 0 
        # xp    -> 1 
        # y     -> 2 
        # yp    -> 3 
        # tau   -> 4 
        # delta -> 5 
                           
        var_dict = {"x":0, "xp":1, "y":2, "yp":3, "tau":4, "delta":5}
        scale = [1e6, 1e6, 1e6, 1e6, 1e12, 1e6]
        label_list = ["x (um)", "x' ($\\mu$rad)", "y (um)", "y' ($\\mu$rad)",
                      "$\\tau$ (ps)", "$\\delta(\\times 10^{-6})$"]
        
        y_var = file[group][dataset][var_dict[option]]*scale[var_dict[option]]
        label = label_list[var_dict[option]]
        
            
    plt.plot(file[group]["time"][:],y_var)
    plt.xlabel("number of turns")
    plt.ylabel(label)
    
    file.close()
            
def plot_phasespacedata(filename, bunch_number, total_size, save_every, dataset, 
                        x_var=None, y_var=None, turn=None, only_alive=False):
    """
    Plot data from PhaseSpaceData_0 group of the HDF5 file.

    Parameters
    ----------
    filename : str
        Name of the HDF5 file that contains the data.
    bunch_number : int
        Number of the bunch whose data has been saved in the HDF5 file.
        This has to be identical to 'bunch_number' parameter in 'PhaseSpaceMonitor' object.
    total_size : int
        Total size of the save regarding to 'PhaseSpaceMonitor' object.
    save_every : int
        Frequency of the save regarding to 'PhaseSpaceMonitor' object.
    dataset : str {'alive', 'partcicles'}
        HDF5 file's dataset to be plotted.
    x_var, y_var : str {"x", "xp", "y", "yp", "tau", "delta"}, optional
        If dataset is "particles", the variables to be plotted on the horizontal
        and the vertical axes need to be specified.
    turn : int, optional
        Turn at which the data will be extracted.
    only_alive : bool
        When only_alive is True, only alive particles are plotted and dead particles will be discarded.

    """
    
    file = hp.File(filename, "r")
    
    group = "PhaseSpaceData_{0}".format(bunch_number)
    
    data_points = int(total_size/save_every)
    timelist = file[group]["time"][0:data_points] # example output: [0,10,20,30,40]
    
    if dataset == "alive":
        alive_at_a_time = []
        for i in range (data_points):
            alive_at_a_time.append(np.sum(file[group][dataset][:,i]))
            
        plt.plot(timelist,alive_at_a_time)
        plt.xlabel("number of turns")
        plt.ylabel("number of alive particles")
        
    elif dataset == "particles":
        # Specify the parameter
        #     x     -> 0 
        #     xp    -> 1 
        #     y     -> 2 
        #     yp    -> 3 
        #     tau   -> 4 
        #     delta -> 5  
             
        var_dict = {"x":0, "xp":1, "y":2, "yp":3, "tau":4, "delta":5}
        scale = [1e3,1e3,1e3,1e3,1e12,1e3]
        label = ["x (mm)","x' (mrad)","y (mm)","y' (mrad)","$\\tau$ (ps)","$\\delta \\times 10^{-3}$"]
        
        # find the index of "turn" in timelist
        index_in_timelist = np.where(timelist==turn) 
        
        path = file[group][dataset]
                        
        if only_alive == False:
            # format : sns.jointplot(x_axis, yaxis, kind)
            x_axis = path[:,var_dict[x_var],index_in_timelist]
            y_axis = path[:,var_dict[y_var],index_in_timelist]


        elif only_alive == True:
            x_alive = []
            y_alive = []
            for i in path[:,var_dict[x_var],index_in_timelist]:
                if bool(i) == True:
                    x_alive.append(i)
                else: pass
            for i in path[:,var_dict[y_var],index_in_timelist]:
                if bool(i) == True:
                    y_alive.append(i)
                else: pass
            x_axis = np.array(x_alive)
            y_axis = np.array(y_alive)
            
        sns.jointplot(x_axis*scale[var_dict[x_var]], y_axis*scale[var_dict[y_var]],
                      kind="kde")
        
        plt.xlabel(label[var_dict[x_var]])
        plt.ylabel(label[var_dict[y_var]])
        
            
    file.close()

    
    
     
    
